#!/usr/bin/python
#-*- coding:utf-8 -*-
import Get_BME280
from time import sleep
import datetime
import time
import locale
import matplotlib as mplt
mplt.use("Agg")
import matplotlib.dates as mdate
import matplotlib.pyplot as plt
import csv
import os
#ディレクトリ指定
logpath = '/home/pi/Logger/'
now = datetime.datetime.now()
filename = "Log_"+now.strftime("%Y-%m") 
graphfile = 'graph.png'
#**** グラフ生成関数 ****
def makeGraph():
    #ファイルからデータの読み出し
    f = open(logpath + filename+'.csv', 'r')
    dates = []
    temp = []
    humi = []
    pres = []
    #ファイルからデータ取り出し
    for line in csv.reader(f):
        dates.append(datetime.datetime.strptime(line[0], "%m%d%H%M"))
        pres.append(float(line[1]))
        temp.append(float(line[2]))
        humi.append(float(line[3]))                  
    f.close()
    #時刻データの数値への変換
    x = mplt.dates.date2num(dates)       #数値へ変換
    #グラフのインスタンス取得
    fig, ax1 = plt.subplots(1,1, sharex=True, figsize=(12, 6))
    plt.ylim([0, 80])           #ax1のY軸スケール指定
    ax2 = ax1.twinx()
    plt.ylim([960, 1040])       #ax2のY軸スケール指定
    #データグラフ描画 X軸を時間にする
    ax1.plot_date(x, temp, 'r', xdate=True, label="Temperature")    #color=red Time
    ax1.plot_date(x, humi, 'g', xdate=True, label="Humidity")       #color=green
    ax2.plot_date(x, pres, 'b', xdate=True, label="Pressure")       #color=blue
    #横軸の補助線追加
    ax1.axhline(y=20, color='c', linestyle='--')
    ax1.axhline(y=40, color='c', linestyle='--')
    ax1.axhline(y=60, color='c', linestyle='--')
    #時間の表示フォーマット指定
    ax1.xaxis.set_major_formatter(mdate.DateFormatter('%b/%d_%H')) #月日時表示
    ax1.xaxis.set_major_locator(mdate.HourLocator(interval=12))    #12時間ごと
    plt.gcf().autofmt_xdate()                       #斜め表示
    #ラベル類表示  
    ax1.legend(loc='lower left')                    #凡例表示位置指定    
    ax1.set_xlabel('Time')
    ax1.set_ylabel('Temperature (DegC)    Humiduty (%RH)')
    ax2.legend(loc='lower right')     
    ax2.set_ylabel('Pressure (hPa)')
    ax1.set_title('Room Environment', fontsize=25)
    #グラフ保存
    fig.savefig(logpath+graphfile)
    
#**** メイン関数　*****
def main():
    #データの取得
    stamp = "%s" % now.strftime('%m%d%H%M')
    data = Get_BME280.readData()
    #もしフォルダがなければ新規作成する
    if not os.path.exists('/home/pi/Logger'):
        os.makedirs('/home/pi/Logger')    
    #データのファイル保存
    f = open(logpath + filename + '.csv', 'a')
    f.write(stamp+','+data+"\r\n")
    f.close()
    #グラフ作成実行
    makeGraph()

#************* *****************
if __name__=='__main__':
    main()
